#!/usr/bin/python
# Copyright (c) 2009-2010 Digi International Inc., All Rights Reserved
#
# This software contains proprietary and confidential information of Digi
# International Inc.  By accepting transfer of this copy, Recipient agrees
# to retain this software in confidence, to prevent disclosure to others,
# and to make no use of this software other than that for which it was
# delivered.  This is an published copyrighted work of Digi International
# Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
# prohibited.
#
# Restricted Rights Legend
#
# Use, duplication, or disclosure by the Government is subject to
# restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
# Technical Data and Computer Software clause at DFARS 252.227-7031 or
# subparagraphs (c)(1) and (2) of the Commercial Computer Software -
# Restricted Rights at 48 CFR 52.227-19, as applicable.
#
# Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343

USAGE = \
"""usage: %prog [options] idigi_url idigi_username idigi_password gateway_id

idigi_url        The iDigi instance associated with the gateway where the
                 ERT/SE Bridge driver will be installed.
idigi_username   The username of the iDigi user who owns the gateway where the
                 ERT/SE Bridge driver will be installed.
idigi_password   The password for idigi_username.
gateway_id       The last six digits of the MAC address of the gateway where the
                 driver will be installed.
"""

# *** Imports

import httplib
import re
import telnetlib
import time
import xml.etree.ElementTree

from base64 import encodestring as toB64, decodestring as fromB64

# *** Main

def main():

    import optparse
    import re

    GATEWAY_REGEX = "^[A-Fa-f0-9]{6}$"

    optparser = optparse.OptionParser(usage=USAGE)
    optparser.add_option("-f",
                         "--force",
                         action="store_true",
                         dest="force",
                         help="Override a previous installation.",
                         default=False)
    (options, args) = optparser.parse_args()

    # validate arguments
    if len(args) != 4:
        idigi_url = idigi_username = idigi_password = gateway_id = None
        idigi_url = raw_input('Please enter the iDigi instance associated with \
your gateway (e.g. developer.idigi.com) :> ')
        if idigi_url == '':
            optparser.error("No URL entered.")

        idigi_username = raw_input('Please enter the username of the iDigi user \
who owns the gateway :> ')
        if idigi_username == '':
            optparser.error("No username entered.")

        idigi_password = raw_input('Please enter the password of the \
iDigi user who owns the gateway :> ')
        if idigi_password == '':
            optparser.error("No password entered.")       

        gateway_id = raw_input('Please enter the last six digits of \
the gateway ID (e.g. XXXXXX) :> ')
        if gateway_id == '':
            optparser.error("No Gateway ID entered.")
        
        if not re.match(GATEWAY_REGEX, gateway_id):
            optparser.error("Gateway id must be a six digit base 16 string.")
        print "\n"

        args = [idigi_url,idigi_username,idigi_password,gateway_id]
             
    if not re.match(GATEWAY_REGEX, args[3]):
        optparser.error("Gateway id must be a six digit base 16 string.")

    # install
    ERTSEBridgeInstaller(*args).install(force=options.force)

# *** Request builder helper functions

def do_command(target, xml):
    return "<do_command target='%s'>%s</do_command>" % (target, xml)

def do_commands(target, xml_list):
    return do_command(target, "".join(xml_list))

def get_file_command(filename):
    return "<get_file synchronous='true' name='%s'/>" % filename

def put_file_command(filename, data):
    return "<put_file synchronous='true' name='%s'><data>%s</data></put_file>" \
           % (filename, data)


def upload_command():
    return put_file_command("/WEB/python/" +
                            DRIVER_PACKAGE_FILE_NAME,
                            DRIVER_PACKAGE_DATA)
# *** Constants

# Module configuration
ENDPOINT_MODULE_NAME = "RPC_ERT_Config_Interface"
OTA_MODULE_NAME = "RPC_OTA_PXBee_Interface"

# Endpoint configuration
DEFAULT_ENDPOINT_ID = 0xE8
MAX_ENDPOINT_ID = 0xF0
ENDPOINT_CLASS_NAME = "OTA_PXBee_Endpoint"
ENDPOINT_PROFILE_ID = 0xC105
ENDPOINT_DEVICE_ID = 0x0000
ENDPOINT_DEVICE_VERSION = 0x00
ENDPOINT_ID_REGEX = ".* 0x%x 0x\d* 0x\d* 0x\d*"

# Cluster Configuration
CONFIG_CLUSTER_NAME = "ERT_Config_Cluster_Client"
CLUSTER_ENDPOINT_ID = "0x5E"
CLUSTER_SERVER_OR_CLIENT = "Client"
CLUSTER_ID = "0xFE5E"
CLUSTER_ENABLE_APS = "True"

# Interface Configuration
CONFIG_INTERFACE_NAME = "RPC_ERT_Config_Interface"
OTA_INTERFACE_NAME = "RPC_OTA_PXBee_Interface"

# File paths
PATHS_INI_FILE_NAME = "paths.ini"
MODULES_INI_FILE_NAME = "modules.ini"
ENDPOINTS_INI_FILE_NAME = "endpoints.ini"
CLUSTERS_INI_FILE_NAME = "clusters.ini"
INTERFACES_INI_FILE_NAME = "interfaces.ini"

DRIVER_PACKAGE_FILE_NAME = "ERTSE.zip"
SE_FRAMEWORK_PACKAGE_FILE_NAME = "SE_Framework.zip"

BASE_FILE_PATH = "/WEB/python/"
PATHS_INI_FILE_PATH = BASE_FILE_PATH + PATHS_INI_FILE_NAME
MODULES_INI_FILE_PATH = BASE_FILE_PATH + MODULES_INI_FILE_NAME
ENDPOINTS_INI_FILE_PATH = BASE_FILE_PATH + ENDPOINTS_INI_FILE_NAME
CLUSTERS_INI_FILE_PATH = BASE_FILE_PATH + CLUSTERS_INI_FILE_NAME
INTERFACES_INI_FILE_PATH = BASE_FILE_PATH + INTERFACES_INI_FILE_NAME

# XML Commands

RCI_TEMPLATE = """<sci_request version="1.0">
  <send_message>
    <targets>
      <device id="00000000-00000000-00409DFF-FF%s" />
    </targets>
    <rci_request version="1.1">%s</rci_request>
  </send_message>
</sci_request>
"""

GET_FILES_COMMAND = "<ls dir='/WEB/python'/>"

# XML paths

LS_XML_PATH = "send_message/device/rci_reply/do_command/ls/file"
IP_XML_PATH = "send_message/device/rci_reply/query_state/boot_stats/ip"
FILE_XML_PATH = "send_message/device/rci_reply/do_command/get_file"

# *** Class definitions

class RciManager:
    """Manages sending and receiving messages from iDigi."""

    def __init__(self, idigi_instance_url, username, password, gateway_id):
        self._idigi_instance_url = idigi_instance_url
        self._username = username
        self._password = password
        self._gateway_id = gateway_id
        self._conn = httplib.HTTPConnection(idigi_instance_url + ":80")


        authstring = "Basic " + \
            toB64(self._username + ":" + self._password).strip()
        self._headers = {"Content-type": "text/xml",
                        "Accept": "text/xml",
                        "Authorization": authstring}

    def send_command_and_read(self, command, target="RPC_request"):
        return self.request_and_read(do_command(target, command))

    def send_commands_and_read(self, commands, target="RPC_request"):
        return self.request_and_read(do_commands(target, commands))

    def request_and_read(self, rci_request):
        request = RCI_TEMPLATE % (self._gateway_id, rci_request)
        self._conn.request("POST", "/ws/sci", request, self._headers)
        response = self._conn.getresponse()
        assert response.status == 200, "Error communicating with iDigi."
        result = response.read()
        return result

class ERTSEBridgeInstaller:
    """Manages installing the ERT/SE Bridge driver."""

    def __init__(self, idigi_instance_url, username, password, gateway_id,
                 force=False):
        self._force = force
        self._rci_manager = RciManager(idigi_instance_url,
                                       username,
                                       password,
                                       gateway_id)

    def _init_install(self):
        self._has_se_framework = False
        self._has_driver_package = False
        
        self._empty_paths_ini = True
        self._empty_modules_ini = True
        self._empty_endpoints_ini = True
        self._empty_clusters_ini = True
        self._empty_interfaces_ini = True

    def _check_device_configuration(self):
        # Get device IP Address.
        print "Querying device state."

        query_state_raw = self._rci_manager.request_and_read("<query_state/>")
        query_state_elem = xml.etree.ElementTree.fromstring(query_state_raw)
        self._current_device_ip = query_state_elem.find(IP_XML_PATH).text
        
            
        # TODO jfocht: This may not be the device IP 100% of the time.
	try:
        	print "Retrieved device IP: %s" % (self._current_device_ip, )
	except:
		print "ERROR: Could not contact iDigi to query device state. \
                Check iDigi URL/username/password for correctness."	
        # Get device files.
        print "Querying device files"
        result = self._rci_manager.send_command_and_read(GET_FILES_COMMAND,
                                                         'file_system')
        result_elem = xml.etree.ElementTree.fromstring(result)
        for e in result_elem.findall(LS_XML_PATH):
            if e.attrib["name"] == SE_FRAMEWORK_PACKAGE_FILE_NAME:
                self._has_se_framework = True
                print "Found " + SE_FRAMEWORK_PACKAGE_FILE_NAME
            elif e.attrib["name"] == DRIVER_PACKAGE_FILE_NAME:
                self._has_driver_package = True
                print "Found " + DRIVER_PACKAGE_FILE_NAME
            elif e.attrib["name"] == PATHS_INI_FILE_NAME:
                self._empty_paths_ini = (e.attrib["size"] == '0')
                print PATHS_INI_FILE_NAME + " is empty? %s" % (
                    self._empty_paths_ini)
            elif e.attrib["name"] == MODULES_INI_FILE_NAME:
                self._empty_modules_ini = (e.attrib["size"] == '0')
                print MODULES_INI_FILE_NAME + " is empty? %s" % (
                    self._empty_modules_ini)
            elif e.attrib["name"] == ENDPOINTS_INI_FILE_NAME:
                self._empty_endpoints_ini = (e.attrib["size"] == '0')
                print ENDPOINTS_INI_FILE_NAME + " is empty? %s" % (
                    self._empty_endpoints_ini)
            elif e.attrib["name"] == CLUSTERS_INI_FILE_NAME:
                self._empty_clusters_ini = (e.attrib["size"] == '0')
                print CLUSTERS_INI_FILE_NAME + " is empty? %s" % (
                    self._empty_clusters_ini)
            elif e.attrib["name"] == INTERFACES_INI_FILE_NAME:
                self._empty_interfaces_ini = (e.attrib["size"] == '0')
                print INTERFACES_INI_FILE_NAME + " is empty? %s" % (
                    self._empty_interfaces_ini)
        assert self._has_se_framework, \
            "The ERTSEBridge driver requires the SE Framework."

    def _configure_gateway(self):
        print "Configuring gateway."

        # download non-empty configuration files
        print "Downloaded configuration files: "
        ini_files_to_download = []
        if not self._empty_paths_ini:
            print "\t" + PATHS_INI_FILE_NAME
            ini_files_to_download.append(PATHS_INI_FILE_PATH)
        if not self._empty_modules_ini:
            print "\t" + MODULES_INI_FILE_NAME
            ini_files_to_download.append(MODULES_INI_FILE_PATH)
        if not self._empty_endpoints_ini:
            print "\t" + ENDPOINTS_INI_FILE_NAME
            ini_files_to_download.append(ENDPOINTS_INI_FILE_PATH)
        if not self._empty_clusters_ini:
            print "\t" + CLUSTERS_INI_FILE_NAME
            ini_files_to_download.append(CLUSTERS_INI_FILE_PATH)
        if not self._empty_interfaces_ini:
            print "\t" + INTERFACES_INI_FILE_NAME
            ini_files_to_download.append(INTERFACES_INI_FILE_PATH)
            
        get_ini_file_cmds = [get_file_command(f) for f in ini_files_to_download]
        result = self._rci_manager.send_commands_and_read(get_ini_file_cmds,
                                                          "file_system")

        # process the ini files
        print "Processing ini files"
        update_paths_ini = update_modules_ini = update_endpoints_ini = False
        update_clusters_ini = update_interfaces_ini = False
        paths_ini = modules_ini = endpoints_ini = ''
        clusters_ini = interfaces_ini = ''
        result_elem = xml.etree.ElementTree.fromstring(result)
        for e in result_elem.findall(FILE_XML_PATH):
            if e.attrib["name"] == PATHS_INI_FILE_PATH:
                paths_ini = fromB64(e.find('data').text)
            elif e.attrib["name"] == MODULES_INI_FILE_PATH:
                modules_ini = fromB64(e.find('data').text)
            elif e.attrib["name"] == ENDPOINTS_INI_FILE_PATH:
                endpoints_ini = fromB64(e.find('data').text)
            elif e.attrib["name"] == CLUSTERS_INI_FILE_PATH:
                clusters_ini = fromB64(e.find('data').text)
            elif e.attrib["name"] == INTERFACES_INI_FILE_PATH:
                interfaces_ini = fromB64(e.find('data').text)

        # build the new configuration text
        
        if paths_ini.find(DRIVER_PACKAGE_FILE_NAME) < 0:
            update_paths_ini = True
            paths_ini = _add_newline_if_not_empty(paths_ini)
            paths_ini += DRIVER_PACKAGE_FILE_NAME
            
        if modules_ini.find(ENDPOINT_MODULE_NAME) < 0:
            update_modules_ini = True
            modules_ini = _add_newline_if_not_empty(modules_ini)
            modules_ini += ENDPOINT_MODULE_NAME
            modules_ini += "\n"
            modules_ini += OTA_MODULE_NAME
            modules_ini += "\n"
            
        if endpoints_ini.find(ENDPOINT_CLASS_NAME) < 0:
            # Starting at the default endpoint id, scan for the next
            # available endpoint id.
            endpoint_id = DEFAULT_ENDPOINT_ID
            while (re.search(ENDPOINT_ID_REGEX % endpoint_id, endpoints_ini) and
                   endpoint_id <= MAX_ENDPOINT_ID):
                endpoint_id += 1
            assert endpoint_id <= MAX_ENDPOINT_ID, \
                "No endpoint available for %s." % ENDPOINT_CLASS_NAME
            update_endpoints_ini = True
            endpoints_ini = _add_newline_if_not_empty(endpoints_ini)
            endpoints_ini += " ".join((ENDPOINT_CLASS_NAME,
                                       hex(endpoint_id),
                                       hex(ENDPOINT_PROFILE_ID),
                                       hex(ENDPOINT_DEVICE_ID),
                                       hex(ENDPOINT_DEVICE_VERSION)))
            endpoints_ini += "\n"  
        
        if clusters_ini.find(CONFIG_CLUSTER_NAME) < 0:
            update_clusters_ini = True
            clusters_ini = _add_newline_if_not_empty(clusters_ini)
            clusters_ini += " ".join((CONFIG_CLUSTER_NAME, 
                                     CLUSTER_ENDPOINT_ID,
                                     CLUSTER_SERVER_OR_CLIENT,
                                     CLUSTER_ID,
                                     CLUSTER_ENABLE_APS))
            clusters_ini += "\n"   
                            
        if interfaces_ini.find(CONFIG_INTERFACE_NAME) < 0:
            update_interfaces_ini = True
            interfaces_ini = _add_newline_if_not_empty(interfaces_ini)
            interfaces_ini += CONFIG_INTERFACE_NAME
            interfaces_ini += "\n"
            interfaces_ini += OTA_INTERFACE_NAME
            interfaces_ini += "\n"
                
        # upload the new configuration files
        print "Uploading new ini files:"
        commands = []
        if update_paths_ini:
            print "\t" + PATHS_INI_FILE_PATH
            commands.append(put_file_command(PATHS_INI_FILE_PATH,
                                             toB64(paths_ini)))
        if update_modules_ini:
            print "\t" + MODULES_INI_FILE_PATH
            commands.append(put_file_command(MODULES_INI_FILE_PATH,
                                             toB64(modules_ini)))
        if update_endpoints_ini:
            print "\t" + ENDPOINTS_INI_FILE_PATH
            commands.append(put_file_command(ENDPOINTS_INI_FILE_PATH,
                                             toB64(endpoints_ini)))
        if update_clusters_ini:
            print "\t" + CLUSTERS_INI_FILE_PATH
            commands.append(put_file_command(CLUSTERS_INI_FILE_PATH,
                                             toB64(clusters_ini)))
        if update_interfaces_ini:
            print "\t" + INTERFACES_INI_FILE_PATH
            commands.append(put_file_command(INTERFACES_INI_FILE_PATH,
                                             toB64(interfaces_ini)))
        if commands:
            status = self._rci_manager.send_commands_and_read(commands,
                                                              "file_system")

        # TODO jfocht: Verify configuration files were uploaded successfully.

   
    def _upload_ertse_driver(self):
        print "Uploading " + DRIVER_PACKAGE_FILE_NAME
        result = self._rci_manager.send_command_and_read(upload_command(),
                                                         "file_system")

        # TODO jfocht: Verify ERTSE.zip was uploaded successfully.

    def _reboot_device(self):
        print "Rebooting device"
        net = telnetlib.Telnet(self._current_device_ip)
        net.read_until("#>")
        net.write("boot action=reset\n")
        net.close()

    def install(self,force=False):
        self._init_install()
        self._check_device_configuration()
        if not self._has_driver_package or force:
            self._upload_ertse_driver()
        self._configure_gateway()
        self._reboot_device()
        test = raw_input('===Finished (Press Enter to Exit)===')

# *** Helper functions

def _add_newline_if_not_empty(text):
    if text:
        text += "\n"
    return text

# This will be replaced when the code is built.

DRIVER_PACKAGE_DATA = """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"""


# *** Main

if __name__ == "__main__":
    main()
